%% Significance Analysis of Microarrays implementation using Matlab
%
% The demonstration proceeds to a standard MatSAM analysis. To see the
% complete documentation for the functions used in the demo, please refer
% to the <index.html API reference home page>.

%% 1- Global arguments
% Defines the names of global variables and sets their values.

global data

fprintf ('Declaring and defining global arguments ...\n\n');
global_args ();

%% 2- The data set
% Specifies the data set from which the experiments and the observations
% data are extracted. The data set is expected to be provided as two text
% files (tabulation separated), the first containing only gene expression
% data values, the second comprised of gene names/tags. This demo is based
% on the Platinum Spike data set (Zhu et al, 2010).
exprs_file = 'inputs/gse21344_expressions.txt';
obs_file = 'inputs/gse21344_names.txt';
fprintf ('Reading into raw files ...\n');
sam_input (exprs_file, obs_file);

%% 3- Experiment groups
% Sets up the design following which the experiments are divided into
% classes. The experiments are divided into two groups (or classes). In the
% dataset here used, there are 18 experiments, the first 9 belong to one
% class (the control group) while the 9 last ones belong to another class
% (the treatement group). To specify groups, we use 0s and 1s labels. Thus,
% the classes design will be the following :
% 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1.
cl = ones (1, size (data, 2));
k = 1;
l = 9;
cl (k:l) = 0;
disp(cl);

%% 4- The main SAM function
% The function gives results related, including but not limited to, the
% expected & observed D scores, the cut-off helping to find expressed &
% false called observations, the scores of the statistical test, and the
% fold changes.
fprintf ('Beginning SAM ...\n\n');
[d, d_bar, vec_false, p_value, q_value, fold_out, s] = sam_func(cl);

%% 5- Thresholds
% Gives the watersheds from which the user will determine how many significant
% observations to pick up.
fprintf ('Summary of obtained results :\n');
full_results (d, d_bar, vec_false);

%% 6- SAM results
% Sets the delta value using the precendetely printed thresholds. The up &
% down regulated observations' statistics are computed based on the chosen
% delta value.
delta = 16.7;
fprintf ('Gathering SAM results ...\n');

[mat_sig, mat_sig_hdr, called_names, mat_fdr] =...
    sam_results (delta, d, d_bar, vec_false, s, p_value, q_value, fold_out);

%% 7- Plotting
% Plots SAM QxQ plot.
fprintf ('Plotting the SAM results ...\n');
sam_plot (d, d_bar, delta, vec_false);

%% 8- Printing
% Saves the results on the disk for further analysis purpose.
fprintf ('Printing the SAM results ...\n\n');

fold_status = mkdir ('results');
if fold_status == 1
    file_name = sprintf ('results/golub_results__delta_%.1f.txt', delta);
    save_sam_results (...
        mat_sig, mat_sig_hdr, mat_fdr, called_names, delta, file_name);
else
    fprintf ('Results not saved; check destination folder settings.\n');
end

%% 9- References
%
% * Virginia Goss Tusher, Robert Tibshirani, Gilbert Chu (2001).
% Significance analysis of microarrays applied to the ionizing radiation
% response. Proceedings of the National Academy of Sciences of USA, 98(9):
% 5116–5121.
%
% * Zhu Q, Miecznikowski JC, Halfon MS (2010). Preferred analysis methods
% for Affymetrix GeneChips. II. An expanded, balanced, wholly-defined
% spike-in dataset. BMC Bioinformatics, 11:285.

%% 10- Authors contact
%
% <mailto:eric.nimpaye@gmail.com Eric NIMPAYE>
%
% <mailto:kaissi.ouafae@gmail.com Ouafae KAISSI>
%
% <mailto:tiratharaj@gmail.com Tiratha Raj SINGH>
%
% <mailto:brigitte.vannier@univ-poitiers.fr Brigitte VANNIER>
%
% <mailto:azeibrah@yahoo.com Azeddine IBRAHIMI>
%
% <mailto:amoussa@uae.ac.ma Ahmed MOUSSA>
