function [d_bar, p_value, vec_false] = d_null (d, type_mt)
%% D_NULL computes D-bar scores and P-values after statistical testing the data.
%
% D_BAR = D_NULL (D, TYPE_MT) does a TYPE_MT statistical test on the data
% and computes the SAM D_BAR scores based on the D scores.
%
% [D_BAR, P_VALUE] = D_NULL (D, TYPE_MT) returns the P_VALUEs of the
% TYPE_MT statistical test.
%
% [D_BAR, P_VALUE, VEC_FALSE] = D_NULL (D, TYPE_MT) gives the thresholds from
% which positive and negative D scores will be rated as false positives.
%
% VALUE
%       D_BAR : expected expression scores.
%       P_VALUE : p-values.
%       VEC_FALSE : limens helping to choose D scores of expressed observations.

global data mat_samp med n_subset WILC_SIGNED_RANK_STAT...
    WILC_SIGNED_RANK_SCORES F_STAT

if med == 1
    n_subset = 1;
end

[n_samp, n_col] = size (mat_samp);

buff_seq = 1:n_subset:n_samp;
buff_seq = [buff_seq, n_samp+1];

seq_samp = unique (buff_seq);

n_int = length (seq_samp) - 1;

n_row = size (data, 1);

d_mat = zeros (n_row, n_int);
mat_neg = zeros (n_row, n_int);
mat_pos = zeros (n_row, n_int);

if (strcmp (type_mt, WILC_SIGNED_RANK_STAT) == 1) ||...
        (strcmp (type_mt, WILC_SIGNED_RANK_SCORES) == 1)
    le_cl = n_col / 2;
    
    data = data (:, 2 .* (1:le_cl)) - data (:, 2 .* (1:le_cl) - 1);
    mat_samp = mat_samp (:, 2 .* (1:le_cl));
end

minus_abs_d = - abs (d);
d_rank = ranking (minus_abs_d);

seq_on_rows = 1:n_row;
d_flat = reshape (d, 1, size (d, 1) * size (d, 2));

for j = 1:n_int
    fprintf ('\n\t...');
    buff_mat = mat_samp (seq_samp (j):(seq_samp (j+1) - 1), :);
    
    dperm_out = build_dperm (data, buff_mat, type_mt, seq_samp);
    
    if sum (sum (isnan (dperm_out))) ~= 0
        dperm_out = purge_nan (dperm_out);
    end
    
    d_mat (:, j) = mean (dperm_out, 2) * size (buff_mat, 1);
    
    %positive matrix
    a_x = sum (sum (dperm_out >= 0)) + seq_on_rows;
    
    buff_abs = abs (d_flat);
    
    buff_pos = dperm_out (dperm_out >= 0);
    
    dperm_flat_pos = reshape (buff_pos, 1,...
        size (buff_pos, 1) * size (buff_pos, 2));
    
    a_y = - [dperm_flat_pos, buff_abs];
    
    buff_ranks = ranking (a_y);
    
    dump_ord = buff_ranks (a_x);
    
    mat_pos (:, j) = dump_ord' - d_rank;
    
    %negative matrix
    a_x = sum (sum (dperm_out < 0)) + seq_on_rows;
    
    buff_abs = -abs (d_flat);
    
    buff_neg = dperm_out (dperm_out < 0);
    
    dperm_flat_neg = reshape (buff_neg, 1,...
        size (buff_neg, 1) * size (buff_neg, 2));
    
    a_y = - [dperm_flat_neg, buff_abs];
    
    buff_ranks = ranking (a_y);
    
    dump_ord = buff_ranks (a_x);
    
    mat_neg (:, j) = dump_ord' - d_rank;
end
fprintf ('\n\n\n');

B = size (mat_samp, 1);

d_bar = sum (d_mat, 2) / B;

p_value = (sum (mat_pos, 2) + sum (mat_neg, 2)) / (n_row * B);

vec_false = zeros (n_row, 1);

if med == 1
    vec_false (d >= 0) = median (mat_pos (d >= 0, :), 2);
    
    if strcmp (type_mt, F_STAT) == 1
        vec_false (d < 0) = median (mat_neg (d < 0, :), 2);
    end
else
    vec_false (d >= 0) = sum (mat_pos (d >= 0, :), 2) / B;
    
    if strcmp (type_mt, F_STAT) == 1
        vec_false (d < 0) = sum (mat_neg (d < 0, :), 2) / B;
    end
end
end
