function [d, d_bar, p_value, vec_false, fold_out, s] = d_stat_func (cl)
%% D_STAT_FUNC corrects the D-scores using a fudge factor.
%
% D = D_STAT_FUNC (CL) applies a statistical test on the data divided in
% classes following a CL design, and corrects the denominator of the result
% by a fudge factor, resulting in SAM D-scores.
%
% [D, D_BAR] = D_STAT_FUNC (CL) returns the D_BAR scores.
%
% [D, D_BAR, P_VALUE] = D_STAT_FUNC (CL) returns the P_VALUEs of the
% statistical test.
%
% [D, D_BAR, P_VALUE, VEC_FALSE] = D_STAT_FUNC (CL) returns cut-off
% VEC_FALSE from which false called observations will be found.
%
% [D, D_BAR, P_VALUE, VEC_FALSE, FOLD_OUT] = D_STAT_FUNC (CL) returns the
% fold changes FOLD_OUT .
%
% [D, D_BAR, P_VALUE, VEC_FALSE, FOLD_OUT, S] = D_STAT_FUNC (CL) returns the
% denominator S of the statistical test.
%
% VALUE
%       D : observed expression scores.
%       D_BAR : expected expression scores.
%       P_VALUE : p-values of the statistical test.
%       VEC_FALSE : limens helping to choose D scores of expressed observations.
%       FOLD_OUT : fold changes.
%       S : denominator of the statistical test.

global data gene_names R_fold na_replace   s0...
    WILC_RANK_SUMS_t WILC_RANK_SUMS_t_equalvar

if ~isnumeric  (data) == 1
    error ('d_stat:wrong_input', 'data must contain numeric values.\n');
end

n_genes = size (data, 1);
final_hand = 0;

% NaNs handling
for i = 1:n_genes
    buff_gene = data (i, :);
    
    if sum (isnan (buff_gene)) ~= 0
        if na_replace == 1
            data (i, :) = purge_nan (buff_gene);
        else
            data (i, :) = [];
        end
        
        buff_msg = strcat ('The gene %s (%i-th gene) is excluded because',...
            ' of its NaNs in some experiments.\n\n');
        warning ('NaN:replace', buff_msg, gene_names{i}, i);
        
        gene_names{i} = [];
        
        final_hand = 1;
    end
end

if final_hand == 1
    [b_nrow, b_ncol] = size (data);
    fprintf (strcat ('New data dimensions :\n\tobservations count :',...
        ' %i\n\texperiments count : %i\n\n'), b_nrow, b_ncol);
end

fprintf ('Deciding which statistical test to apply on the data ...\n\n');
type_mt = test_type (cl);

if (strcmp (type_mt, WILC_RANK_SUMS_t_equalvar) == 0) &&...
        (strcmp (type_mt, WILC_RANK_SUMS_t) == 0)
    R_fold = 0;
    %No gene is rejected based on its fold change.
end

if R_fold > 0
    fprintf ('Computing the genes fold changes ...\n\n');
    mat_fold = get_fold_changes (cl);
    
    n_fulfill = sum (mat_fold (:, 2));
    
    if n_fulfill == 0
        error ('fold:check', strcat ('None of the variables has a fold',...
            ' change larger than %g.\n\n', R_fold));
    end
    
    if n_fulfill < 20
        error ('fold:check', strcat ('Less than 20 variables have a fold',...
            ' change larger than %g.\n\n', R_fold));
    end
    
    fold_out = mat_fold (:, 1);
    
    if n_fulfill < size (mat_fold, 1)
        exclud_fc = find(mat_fold (:, 2) == 0);
        
        fold_out (exclud_fc) = [];
        data (exclud_fc, :) = [];
        
        warning ('fold:check', strcat ('The following genes are excluded',...
            ' on the basis of their low fold changes : \n'));
        
        fprintf ('\n\tRow\tName\n');
        for k = 1:length (exclud_fc)
            fprintf ('\t%i\t%s\n', exclud_fc (k), gene_names {exclud_fc (k)});
        end
        fprintf ('\n');
        
        gene_names (mat_fold (:, 2) == 0) = [];
    end
else
    fold_out = 0;
end

fprintf (strcat ('Computing the numerators and the denominators of the',...
    ' statistical test ...\n\n'));
[r, s, fold_out] = statest_num_denum (cl, type_mt, fold_out);

if isnan (s0) == 1
    fprintf ('Computing the fudge factor, "S0" ...\n\n');
    s0 = fudge_factor (r, s);
end

fprintf ('Computing the D-scores ...\n\n');
d = r ./ (s + s0);

fprintf ('Randomly permuting the experiments ...\n');
setup_mat_samp (cl, type_mt);

fprintf ('Computing the D-bar scores and the P-Values ...\n\n');
[d_bar, p_value, vec_false] = d_null (d, type_mt);

if size (data, 1) ~= n_genes
    p_value = p_value (~ isnan (p_value));
    
    vec_false = vec_false (~ isnan (vec_false));
    
    d = d (~ isnan (d));
    
    if R_fold > 0
        fold_out = fold_out (~ isnan (fold_out));
    end
end
end
