function [r, s, fold_out] = statest_num_denum (cl, type_mt, fold_out)
%% STATEST_NUM_DENUM gives the numerator and the denominator of a statistical test.
%
% R = STATEST_NUM_DENUM (CL, TYPE_MT, FOLD_OUT) returns the numerator R of
% the TYPE_MT statistical test, and removes, if any, all NaNs in FOLD_OUT,
% DATA and GENE_NAMES (the two last being global arguments).
%
% [R, S] = STATEST_NUM_DENUM (CL, TYPE_MT, FOLD_OUT) returns the denominator
% S of the TYPE_MT statistical test.
%
% [R, S, FOLD_OUT] = STATEST_NUM_DENUM (CL, TYPE_MT, FOLD_OUT) returns the
% given fold changes FOLD_OUT with all NaNs, if any, in it removed.
%
% VALUE
%       R : the numerator of the statistical test.
%
%       S : the denominator of the statistical test.
%
%       FOLD_OUT : the fold changes of the observations.

global data gene_names R_fold WILC_SIGNED_RANK_STAT WILC_RANK_SUMS_t...
    WILC_RANK_SUMS_t_equalvar WILC_SIGNED_RANK_SCORES F_STAT

if strcmp (type_mt, WILC_RANK_SUMS_t) == 1
    cl_ids = unique (cl);

    % Row means
    grp1 = data (:, cl == cl_ids (1));
    grp2 = data (:, cl == cl_ids (2));
    avg_grp1 = mean (grp1, 2);
    avg_grp2 = mean (grp2, 2);
    
    %The numerator of the test
    r = avg_grp2 - avg_grp1;
    
    %The denominator of the test
    c1 = size (grp1, 2);
    c2 = size (grp2, 2);
    
    t0 = c1 * (c1 - 1);
    t1 = c2 * (c2 - 1);
    
    sz = size (data, 1);
    s = zeros (sz, 1);
    
    for i = 1:sz
        devi_1 = sum (power (grp1 (i, :) - avg_grp1 (i), 2));
        devi_2 = sum (power (grp2 (i, :) - avg_grp2 (i), 2));
        s (i) = sqrt ((devi_1 / t0) + (devi_2 / t1));
    end
    
    % Exclude genes wich have null variance
    if sum (s == 0) ~= 0
        var0_genes = find (s == 0);
        r (var0_genes) = [];
        s (var0_genes) = [];
        data (var0_genes, :) = [];
        
        if R_fold > 0
            fold_out (var0_genes) = [];
        end
        
        warning ('variance:check', strcat ('The following genes are',...
            ' excluded on the basis of their low variances : \n'));
        
        fprintf ('\n\tRow\tName\n');
        for k = 1:length (var0_genes)
            fprintf ('\t%i\t%s\n', var0_genes (k), gene_names {var0_genes (k)});
        end
        fprintf ('\n');
        
        gene_names (var0_genes) = [];
        
        [b_nrow, b_ncol] = size (data);
        fprintf (strcat ('New data dimensions :\n\tobservations count :',...
            ' %i\n\texperiments count : %i\n\n'), b_nrow, b_ncol);
    end
    
elseif strcmp (type_mt, F_STAT) == 1
    error ('code_to_come', 'Sorry, F-stat is not yet implemented.\n');
    
elseif (strcmp (type_mt, WILC_SIGNED_RANK_STAT) == 1) ||...
        (strcmp (type_mt, WILC_SIGNED_RANK_SCORES) == 1)
    error ('code_to_come', 'Sorry, Wilc-sign is not yet implemented.\n');
    
elseif strcmp (type_mt, WILC_RANK_SUMS_t_equalvar) == 1
    error ('code_to_come', 'Sorry, Wilc-Rank-Var-Equal unimplemented.\n');
    
else
    error ('statest:unknown_test', 'The required test is not available.\n');
end
end
