function [mat_fdr, mat_fdr_hdr] = stats_cal (d, d_bar, vec_false, delta)
%% STATS_CAL gathers the statistics of the SAM.
%
% * MAT_FDR = STATS_CAL (D, D_BAR, VEC_FALSE, DELTA) determines the
%
%       - delta : thresholds from which expressed observations are computed;
%
%       - prior probability that an observation is expressed;
%
%       - false called observations;
%
%       - the False Discovery Rate (FDR).
%
% All those values are stacked into MAT_FDR ; they are computed based on the
% D (observed) scores, D_BAR (expected) scores, the limens of falsely called
% observations VEC_FALSE, and the preferred values of DELTA to include in
% the findings.
%
% * [MAT_FDR, MAT_FDR_HDR] = STATS_CAL (D, D_BAR, VEC_FALSE, DELTA) returns
% the header MAT_FDR_HDR of the statistics container MAT_FDR.
%
% VALUE
%       MAT_FDR : the differential expressed observations' metrics.
%
%       MAT_FDR_HDR : the metric container's header.

global p0 le_delta

d_sort = sort (d);
d_bar_sort = sort (d_bar);

d_diff = (d_sort - d_bar_sort)';

m = length (d_diff);

if isempty (delta) || (length (delta) == 1 && delta == 0)
    buff_absd = abs (d_diff);
    buff_inf = min (buff_absd);
    buff_sup = max (buff_absd);
    
    buff_inf = max (0.1, buff_inf);
    buff_sup = max (1, buff_sup);
    
    buff_step = (buff_sup - buff_inf) / (le_delta - 1);
    
    buff_seq = buff_inf:buff_step:buff_sup;
    
    delta = round (buff_seq * 10) / 10;
    
elseif sum (delta <= 0) ~= 0
    error ('fdr:wrong_delta', 'delta must be larger than 0.\n\n');
else
    le_delta = length (delta);
end

buff_dmp = d_bar_sort (d_bar_sort >= 0);
if isempty (buff_dmp)
    error ('stat_cal:no_gene', 'No gene statistics for delta = %g.\n', delta);
end
buff_dpos = find (d_bar_sort == min (buff_dmp (1)));

j0 = buff_dpos (1);

mat_fdr_hdr = {'Delta', 'p0', 'False', 'Called', 'FDR', 'cutlow', 'cutup',...
    '-thresh', '+thresh'};

mat_fdr = zeros (le_delta, 9);

mat_fdr (:, 1) = delta;
mat_fdr (:, 2) = p0;

buff_flat = reshape (d, 1, size (d, 1) * size (d, 2));
buff_ord = ranking (buff_flat);

vec_order = vec_false (buff_ord);
vec_order = vec_order (~ isnan (vec_order));

for i = 1:le_delta
    %j1
    if sum (d_diff (j0:m) >= delta (i)) ~= 0
        buff_pos = find (d_diff (j0:m) >= delta (i));
        j1 = j0 - 1 + min (buff_pos);
    else
        j1 = m + 1;
    end
    mat_fdr (i, 9) = j1;
    
    %cutup
    if j1 ~= m + 1
        mat_fdr (i, 7) = d_sort (j1);
    else
        mat_fdr (i, 7) = Inf;
    end
    
    %j2
    if (sum (d_diff (1 : (j0 - 1)) <= (- delta (i))) ~= 0) && (j0 ~= 1)
        buff_pos = find (d_diff (1 : (j0 - 1)) <= (- delta (i)));
        j2 = max (buff_pos);
    else
        j2 = 0;
    end
    mat_fdr (i, 8) = j2;
    
    %cutlow
    if j2 ~= 0
        mat_fdr (i, 6) = d_sort (j2);
    else
        mat_fdr (i, 6) = -Inf;
    end
    
    %Called
    mat_fdr (i, 4) = m - j1 + 1 + j2;
    
    %False
    if j1 == m + 1
        buff_add = 0;
    else
        buff_add = vec_order (j1);
    end
    if j2 == 0
        buff_add_2 = 0;
    else
        buff_add_2 = vec_order (j2);
    end
    mat_fdr (i, 3) = buff_add + buff_add_2;
    
    %FDR
    buff_denum = max (mat_fdr (i, 4), 1);
    buff_num = p0 * mat_fdr (i, 3);
    mat_fdr (i, 5) = buff_num / buff_denum;
end
end
